@@ -1,24 +1,28 @@
-VLSTAR <- function(y1, x1 = NULL, p = NULL,
-                     m = 2, st = NULL, constant = TRUE,
-                     n.combi = 50, n.iter = 500,
-                     starting = NULL, epsilon = 10^(-3),
-                     exo = FALSE, method = c('ML', 'NLS')){
-  y <- as.matrix(y1)
+VLSTAR <- function(y, exo = NULL, p = 1,
+                     m = 2, st = NULL, constant = TRUE, starting = NULL,
+                     n.combi = NULL, method = c('ML', 'NLS'), n.iter = 500,
+                      epsilon = 10^(-3)){
+  y <- as.matrix(y)
+  x <- exo
+  method <- match.arg(method)
   if (any(is.na(y)))
     stop("\nNAs in y.\n")
-  if(!method %in% c('ML', 'NLS'))
-    stop('Please, provide a valid estimation method')
+  #if(!method %in% c('ML', 'NLS'))
+  #  stop('Please, provide a valid estimation method')
   if(m < 2)
     stop('The number of regimes should be greater than one.')
   if(is.null(st))
     stop('The transition variable must be supplied.')
-  if(is.null(x1)){
+  if(is.null(x)){
     if(length(y[,1]) != length(st))
       stop('The length of the variables does not match!')
   }else{
-    if(length(y[,1]) != length(as.matrix(x1[,1])) | length(st) != length(as.matrix(x1[,1])) | length(y[,1]) != length(st))
+    if(length(y[,1]) != length(as.matrix(x[,1])) | length(st) != length(as.matrix(x[,1])) | length(y[,1]) != length(st))
       stop('The length of the variables does not match!')
   }
+if(is.null(starting) & is.null(n.combi)){
+  stop('Starting values should be provided or estimated.')
+}
 
   if(is.null(p) | p < 1){
     stop('Please, specify a valid lag order.')
@@ -32,7 +36,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
   }
   colnames(y) <- make.names(colnames(y))
   ##Definition of dimensions, creating variable x with constant
-  yt <- zoo::zoo(y)
+  yt <- zoo(y)
   ylag <- stats::lag(yt, -(1:p))
   ylag <- as.matrix(ylag)
   if(p>1){
@@ -40,21 +44,31 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
   ylag <- ylag[-(1:lagg),]
   }
   y <- y[-c(1:p), ]
-  ncoly <- ncol(y1)
+  ncoly <- ncol(y)
+  if(!is.null(starting)){
+    if(length(starting)!= (m-1)){
+      stop('The length of the list of initial values should be equal to m-1.')
+    }else{
+      if(any(unlist(lapply(starting, ncol))!=2) | any(unlist(lapply(starting, nrow))!=ncoly)){
+        stop('Each element of the starting argument should have two columns and n rows.')
+      }
+
+    }
+  }
   ncolylag <- ncoly*p
-  nrowy <- nrow(y1)
-  ncolx1 <- ncol(x1)
-  const <- rep(1, (nrowy-p))
+  nrowy <- nrow(y)
+  ncolx1 <- ncol(x)
+  const <- rep(1, nrowy)
   if (constant == TRUE){
-    if(exo == TRUE){
-      x1a <- as.matrix(x1[-c(1:p),])
+    if(!is.null(exo)){
+      x1a <- as.matrix(x[-c(1:p),])
       x <- as.matrix(cbind(const,ylag,x1a))
     }else{
       x <- as.matrix(cbind(const,ylag))
     }
     ncolx <- ncol(x)
   }  else{
-    x <- as.matrix(x1)
+    x <- as.matrix(x)
   }
   nrowx <- nrow(x)
   st <- st[(1+p):length(st)]
@@ -63,7 +77,6 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
   param.init$gamma <- rep(1L, ncoly)
   param.init$cg <- colMeans(y)
   q <- ncol(x)-ncolylag
-
   if (is.null(starting)){
     COMBI <- list()
     GAMMA <- list()
@@ -75,8 +88,8 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
       cj <- matrix(ncol = ncoly, nrow = n.combi)
       gamma[1, ] <- param.init$gamma
       cj[1,] <- param.init$c
-      GAMMA[[t]] <- gamma*(1+stats::rnorm(1, mean = 0, sd = 5))
-      CJ[[t]] <- cj*(1+stats::rnorm(1, mean = 0, sd = 5))
+      GAMMA[[t]] <- gamma*(1+rnorm(1, mean = 0, sd = 5))
+      CJ[[t]] <- cj*(1+rnorm(1, mean = 0, sd = 5))
       #Grid for c and gamma
       rangey <- matrix(nrow = ncoly, ncol = 2)
       combi <- list()
@@ -114,19 +127,19 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
             Gt <- diag(glog[i,])
             GT[[t]] <- Gt
           }
-          Gtilde[[i]] <- t(cbind(In, rlist::list.cbind(GT)))
+          Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
           GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
           XX[[i]] <- x[i,] %*%t(x[i,])
           GGXX[[i]] <- kronecker(GG[[i]], XX[[i]])
           XY[[i]] <- x[i, ]%*%t(y[i,])
-          XYG[[i]] <- matrixcalc::vec(XY[[i]]%*%t(Gtilde[[i]]))
+          XYG[[i]] <- vec(XY[[i]]%*%t(Gtilde[[i]]))
           kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
         }
         ggxx <- Reduce(`+`, GGXX)/nrowx
         M <- t(do.call("cbind", kro))
-        Y <- matrixcalc::vec(t(y))
-        Bhat <- MASS::ginv(t(M)%*%M)%*%t(M)%*%Y
-        BB <- ks::invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
+        Y <- vec(t(y))
+        Bhat <- ginv(t(M)%*%M)%*%t(M)%*%Y
+        BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
         resi <- list()
         Ehat <- matrix(NA, ncol = ncoly, nrow = nrowy)
         for (o in 1:nrowx){
@@ -184,20 +197,20 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
       Gt <- diag(glog[i,])
       GT[[t]] <- Gt
     }
-    Gtilde[[i]] <- t(cbind(In, rlist::list.cbind(GT)))
+    Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
     GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
     XX[[i]] <- x[i,] %*%t(x[i,])
     GGXX[[i]] <- kronecker(GG[[i]], XX[[i]])
     XY[[i]] <- x[i, ]%*%t(y[i,])
-    XYG[[i]] <- matrixcalc::vec(XY[[i]]%*%t(Gtilde[[i]]))
+    XYG[[i]] <- vec(XY[[i]]%*%t(Gtilde[[i]]))
     kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
   }
   ggxx <- Reduce(`+`, GGXX)/nrowx
   M <- t(do.call("cbind", kro))
-  Y <- matrixcalc::vec(t(y))
+  Y <- vec(t(y))
   #Estimated coefficients
-  Bhat <- MASS::ginv(t(M)%*%M)%*%t(M)%*%Y
-  BB <- ks::invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
+  Bhat <- ginv(t(M)%*%M)%*%t(M)%*%Y
+  BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
   resi <- list()
   Ehat1 <- matrix(NA, ncol = ncoly, nrow = nrowy)
   for (i in 1:nrowx){
@@ -231,8 +244,8 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
         Gt <- diag(glog[z,])
         GT[[t]] <- Gt
       }
-      Gtilde[[z]] <- t(cbind(In, rlist::list.cbind(GT)))
-      dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%MASS::ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
+      Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
+      dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
     }
     sumdif <- sum(dify)
     logll <- -(nrowy*log(det(Omegahat))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
@@ -260,8 +273,8 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
         Gt <- diag(glog[z,])
         GT[[t]] <- Gt
       }
-      Gtilde[[z]] <- t(cbind(In, rlist::list.cbind(GT)))
-      dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%MASS::ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
+      Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
+      dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
     }
     sumdif <- sum(dify)
     logll <- -(nrowy*log(det(Omegahat))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
@@ -281,8 +294,8 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
   for(t in 1:(m-1)){
     PARAM1[[t]] <- as.data.frame(PARAM[[t]])
   }
-  param <- data.table::rbindlist(PARAM1)
-  param <- matrixcalc::vec(as.matrix(param))
+  param <- rbindlist(PARAM1)
+  param <- vec(as.matrix(param))
 
   message('Maximum likelihood estimation\n')
 
@@ -297,7 +310,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
     #Parameters
     low1 <- replicate(ncoly, 0)
     #1.Maximum likelihood estimation of gamma and c
-    param1 <- stats::optim(par = param, fn = loglike, lower = c(low1, apply(y, 2, min)),
+    param1 <- optim(par = param, fn = loglike, lower = c(low1, apply(y, 2, min)),
                            method="L-BFGS-B")
 
     cgam1 <- matrix(param1$par, ncol = 2, nrow = ncoly)
@@ -321,18 +334,18 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
         Gt <- diag(glog[i,])
         GT[[t]] <- Gt
       }
-      Gtilde[[i]] <- t(cbind(In, rlist::list.cbind(GT)))
+      Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
       GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
       XX[[i]] <- x[i,] %*%t(x[i,])
       XY[[i]] <- x[i, ]%*%t(y[i,])
-      XYOG[[i]] <- matrixcalc::vec(XY[[i]]%*%MASS::ginv(Omegahat)%*%t(Gtilde[[i]]))
-      PsiOmegaPsi[[i]] <- Gtilde[[i]]%*%MASS::ginv(Omegahat)%*%t(Gtilde[[i]])
+      XYOG[[i]] <- vec(XY[[i]]%*%ginv(Omegahat)%*%t(Gtilde[[i]]))
+      PsiOmegaPsi[[i]] <- Gtilde[[i]]%*%ginv(Omegahat)%*%t(Gtilde[[i]])
       kro[[i]] <- kronecker(PsiOmegaPsi[[i]], XX[[i]])
     }
     xyog <- Reduce(`+`, XYOG)/nrowy
     kroxx <- Reduce(`+`, kro)/nrowy
     Bhat <- t(t(xyog)%*%kroxx)
-    BB <- ks::invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
+    BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
     resi <- list()
     fitte <- matrix(nrow = nrowy, ncol = ncoly)
     Ehat1 <- matrix(NA, ncol = ncoly, nrow = nrowy)
@@ -346,14 +359,10 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
     #3. Convergence check
     ll1 <- loglike1(BB, param1$par, Omega = Omegahat)
     if((ll1-ll0)>0){
-      param <- as.matrix(matrixcalc::vec(cgam1))
+      param <- as.matrix(vec(cgam1))
     }
     err <- abs(ll1 - ll0)
     ll0 <- ll1
-    if(iter ==1) {
-      graphics::plot(ll0~1, xlim = c(0, n.iter), ylim = c(0, (ll0+ll0*0.1)))
-    }
-    graphics::points(ll0~iter)
     loglik1[iter] <- ll1
     bbhat[[iter]] <- BB
     omega[[iter]] <- Omegahat
@@ -373,7 +382,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
     bb2[[t]] <- as.data.frame(bbhat[[iter]][,(ncoly*(t-1)+1):(ncoly*t)] +
                                 bbhat[[iter]][,(ncoly*(t)+1):(ncoly*(t+1))])
   }
-  bb4 <- as.matrix(data.table::rbindlist(bb2))
+  bb4 <- as.matrix(rbindlist(bb2))
   BBhat <- rbind(bb1, bb4)
   colnames(cgam1) <- c('gamma', 'c')
   covbb <- matrix(nrow = m*ncolx, ncol = ncoly)
@@ -382,9 +391,9 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
   ee <- matrix(nrow = m*ncolx, ncol = ncoly)
   for (j in 1 : ncoly){
     #covbb[,j] <- diag(ginv(t(x[[j]])%*%XX[[j]])*sqrt(varhat[j]))
-    covbb[,j] <- sqrt(diag(MASS::ginv(t(x) %*%x))*varhat[j])
+    covbb[,j] <- sqrt(diag(ginv(t(x) %*%x))*varhat[j])
     ttest[,j] <- BBhat[,j]/covbb[,j]
-    pval[,j] <- 2*stats::pt(abs(ttest[,j]),df=(nrowy*m-m*(ncolx)), lower = FALSE)
+    pval[,j] <- 2*pt(abs(ttest[,j]),df=(nrowy*m-m*(ncolx)), lower.tail = FALSE)
   }
 
   loglike2 <- function(y, resid1, omega){
@@ -399,7 +408,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
   AIC1 <- NULL
   BIC1 <- NULL
   for (l in 1:ncoly){
-    ll2[l] <- loglike2(y1[,l], residui[,l], omega1[l])
+    ll2[l] <- loglike2(y[,l], residui[,l], omega1[l])
     AIC1[l] <- 2*k - 2*ll2[l]
     BIC1[l] <- -2*ll2[l] + k*log(nrowy)
   }
@@ -407,13 +416,14 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
   for(j in 1:m){
     names1[[j]] <- as.data.frame(paste(colnames(x), 'm_', j))
   }
-  names1 <- as.matrix(data.table::rbindlist(names1))
+  names1 <- as.matrix(rbindlist(names1))
   rownames(BBhat) <- names1
   colnames(BBhat) <- colnames(y)
   modeldata <- list(y, x)
+  fitte <- fitte[!is.na(fitte[,1]),]
   results <- list(BBhat, covbb, ttest, pval, cgam1, omega[[iter]], fitte, residuals1, ll1, ll2, AIC1, BIC1, Gt, modeldata, BB, m, p,
-                  st, y1, exo, constant)
-  names(results) <- c('Bhat','StDev', 'ttest', 'pval', 'Cgamma', 'Omega', 'fitted', 'residuals', 'MultiLL', 'LL', 'AIC',
+                  st, y, exo, constant)
+  names(results) <- c('Bhat','StDev', 'ttest', 'pval', 'Gammac', 'Omega', 'fitted', 'residuals', 'MultiLL', 'LL', 'AIC',
                       'BIC', 'Gtilde', 'Data', 'B', 'm', 'p', 'st', 'yoriginal', 'exo', 'constant')
   }else{
     #NLS Estimation of Bhat and Omegahat to be used in the first iteration of minimizing Qt
@@ -436,19 +446,19 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
         Gt <- diag(glog[i,])
         GT[[t]] <- Gt
       }
-      Gtilde[[i]] <- t(cbind(In, rlist::list.cbind(GT)))
+      Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
       GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
       XX[[i]] <- x[i,] %*%t(x[i,])
       GGXX[[i]] <- kronecker(GG[[i]], XX[[i]])
       XY[[i]] <- x[i, ]%*%t(y[i,])
-      XYG[[i]] <- matrixcalc::vec(XY[[i]]%*%t(Gtilde[[i]]))
+      XYG[[i]] <- vec(XY[[i]]%*%t(Gtilde[[i]]))
       kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
     }
     ggxx <- Reduce(`+`, GGXX)/nrowx
     M <- t(do.call("cbind", kro))
-    Y <- matrixcalc::vec(t(y))
-    Bhat <- MASS::ginv(t(M)%*%M)%*%t(M)%*%Y
-    BB <- ks::invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
+    Y <- vec(t(y))
+    Bhat <- ginv(t(M)%*%M)%*%t(M)%*%Y
+    BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
 
     #Sum of squared error to be optimized
     ssq1 <- function(param){
@@ -471,7 +481,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
           Gt <- diag(glog[z,])
           GT[[t]] <- Gt
         }
-        Gtilde[[z]] <- t(cbind(In, rlist::list.cbind(GT)))
+        Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
         dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
       }
       sumdif <- sum(dify)
@@ -499,8 +509,8 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
           Gt <- diag(glog[z,])
           GT[[t]] <- Gt
         }
-        Gtilde[[z]] <- t(cbind(In, rlist::list.cbind(GT)))
-        dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%MASS::ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
+        Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
+        dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
       }
       sumdif <- sum(dify)
       logll <- -(nrowy*log(det(Omegahat))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
@@ -520,8 +530,8 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
     for(t in 1:(m-1)){
       PARAM1[[t]] <- as.data.frame(PARAM[[t]])
     }
-    param <- as.matrix(data.table::rbindlist(PARAM1))
-    param <- matrixcalc::vec(param)
+    param <- as.matrix(rbindlist(PARAM1))
+    param <- vec(param)
 
     message('NLS estimation\n')
 
@@ -536,7 +546,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
       #Parameters
       low1 <- replicate(ncoly, 0)
       #1.Maximum likelihood estimation of gamma and c
-      param1 <- stats::optim(par = param, fn = ssq1, lower = c(low1, apply(y, 2, min)),
+      param1 <- optim(par = param, fn = ssq1, lower = c(low1, apply(y, 2, min)),
                              method="L-BFGS-B")
 
       cgam1 <- matrix(param1$par, ncol = 2L, nrow = ncoly*(m-1))
@@ -560,19 +570,19 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
           Gt <- diag(glog[i,])
           GT[[t]] <- Gt
         }
-        Gtilde[[i]] <- t(cbind(In, rlist::list.cbind(GT)))
+        Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
         GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
         XX[[i]] <- x[i,] %*%t(x[i,])
         GGXX[[i]] <- kronecker(GG[[i]], XX[[i]])
         XY[[i]] <- x[i, ]%*%t(y[i,])
-        XYG[[i]] <- matrixcalc::vec(XY[[i]]%*%t(Gtilde[[i]]))
+        XYG[[i]] <- vec(XY[[i]]%*%t(Gtilde[[i]]))
         kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
       }
       ggxx <- Reduce(`+`, GGXX)/nrowx
       M <- t(do.call("cbind", kro))
-      Y <- matrixcalc::vec(t(y))
-      Bhat <- MASS::ginv(t(M)%*%M)%*%t(M)%*%Y
-      BB <- ks::invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
+      Y <- vec(t(y))
+      Bhat <- ginv(t(M)%*%M)%*%t(M)%*%Y
+      BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
       resi <- list()
       resiresi <- list()
       fitte <- matrix(nrow = nrowy, ncol = ncoly)
@@ -591,14 +601,10 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
       #3. Convergence check
       ll1 <- loglike1(BB, param1$par, Omega = Omegahat)
       if((ll1-ll0)>0){
-        param <- as.matrix(matrixcalc::vec(cgam1))
+        param <- as.matrix(vec(cgam1))
       }
       err <- abs(ll1 - ll0)
       ll0 <- ll1
-      if(iter ==1) {
-        graphics::plot(ll0~1, xlim = c(0, n.iter), ylim = c(0, (ll0+ll0*0.1)))
-      }
-      graphics::points(ll0~iter)
       loglik1[iter] <- ll1
       bbhat[[iter]] <- BB
       omega[[iter]] <- Omegahat
@@ -628,7 +634,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
       bb2[[t]] <- as.data.frame(bbhat[[iter]][,(ncoly*(t-1)+1):(ncoly*t)] +
                                   bbhat[[iter]][,(ncoly*(t)+1):(ncoly*(t+1))])
     }
-    bb4 <- as.matrix(data.table::rbindlist(bb2))
+    bb4 <- as.matrix(rbindlist(bb2))
     BBhat <- rbind(bb1, bb4)
     colnames(cgam1) <- c('gamma', 'c')
     covbb <- matrix(nrow = m*ncolx, ncol = ncoly)
@@ -636,10 +642,10 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
     pval <- matrix(nrow = m*ncolx, ncol = ncoly)
     ee <- matrix(nrow = m*ncolx, ncol = ncoly)
     for (j in 1 : ncoly){
-      #covbb[,j] <- diag(MASS::ginv(t(x[[j]])%*%XX[[j]])*sqrt(varhat[j]))
-      covbb[,j] <- sqrt(diag(MASS::ginv(t(x) %*%x))*varhat[j])
+      #covbb[,j] <- diag(ginv(t(x[[j]])%*%XX[[j]])*sqrt(varhat[j]))
+      covbb[,j] <- sqrt(diag(ginv(t(x) %*%x))*varhat[j])
       ttest[,j] <- BBhat[,j]/covbb[,j]
-      pval[,j] <- 2*stats::pt(abs(ttest[,j]),df=(nrowy*m-m*(ncolx)), lower = FALSE)
+      pval[,j] <- 2*pt(abs(ttest[,j]),df=(nrowy*m-m*(ncolx)), lower.tail = FALSE)
     }
     signifi <- matrix('',nrow = nrow(pval), ncol = ncol(pval))
     for(i in 1:nrow(pval)){
@@ -674,7 +680,7 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
     AIC1 <- NULL
     BIC1 <- NULL
     for (l in 1:ncoly){
-      ll2[l] <- loglike2(y1[,l], residui[,l], omega1[l])
+      ll2[l] <- loglike2(y[,l], residui[,l], omega1[l])
       AIC1[l] <- 2*k - 2*ll2[l]
       BIC1[l] <- -2*ll2[l] + k*log(nrowy)
     }
@@ -682,15 +688,16 @@ VLSTAR <- function(y1, x1 = NULL, p = NULL,
     for(j in 1:m){
       names1[[j]] <- as.data.frame(paste(colnames(x), 'm_', j))
     }
-    names1 <- as.matrix(data.table::rbindlist(names1))
+    names1 <- as.matrix(rbindlist(names1))
     rownames(BBhat) <- names1
     colnames(BBhat) <- colnames(y)
     rownames(bhat1) <- names1
     colnames(bhat1) <- colnames(y)
     modeldata <- list(y, x)
+    fitte <- fitte[!is.na(fitte[,1]),]
     results <- list(BBhat, covbb, ttest, pval, cgam1, omega[[iter]], fitte, residuals1, ll1, ll2, AIC1, BIC1, Gt, modeldata, BB, m, p,
-                    st, y1, exo, constant)
-    names(results) <- c('Bhat','StDev', 'ttest', 'pval', 'Cgamma', 'Omega', 'fitted', 'residuals', 'MultiLL', 'LL', 'AIC',
+                    st, y, exo, constant)
+    names(results) <- c('Bhat','StDev', 'ttest', 'pval', 'Gammac', 'Omega', 'fitted', 'residuals', 'MultiLL', 'LL', 'AIC',
                         'BIC', 'Gtilde', 'Data', 'B', 'm', 'p', 'st', 'yoriginal', 'exo', 'constant')
 
   }
@@ -707,8 +714,8 @@ print.VLSTAR <- function(x, ...) {
   order.H <- object$m
   lowCoef <- object$Bhat[grep(paste("m_ ", order.L, sep=''), rownames(object$Bhat)),]
   highCoef <- object$Bhat[grep(paste("m_ ", order.H, sep=''), rownames(object$Bhat)),]
-  gammaCoef <- object$Cgamma[,1]
-  cCoef <- object$Cgamma[,2]
+  gammaCoef <- object$Gammac[,1]
+  cCoef <- object$Gammac[,2]
 
   cat("Coefficients:\n")
   cat("Low regime:\n")
@@ -721,92 +728,3 @@ print.VLSTAR <- function(x, ...) {
   invisible(object)
   NextMethod('print')
 }
-
-#' @S3method summary VLSTAR
-summary.VLSTAR<-function(object,...){
-  x<-object
-  k<-ncol(x$Data[[2]])
-  t<-nrow(x$Data[[1]])
-  p<-x$p
-  x$T <- nrow(x$yoriginal)
-  Z<-t(as.matrix(utils::tail.matrix(x$Data[[1]])))
-  x$npar <- k*ncol(x$Data[[1]])*x$m + 2*(x$m-1)*ncol(x$Data[[1]])
-
-  ## export results
-  x$coefficients<-as.list(as.data.frame(x$Bhat))
-  x$StDev<-as.list(as.data.frame(x$StDev))
-  x$Pvalues<-as.list(as.data.frame(x$pval))
-  x$Tvalues<-as.list(as.data.frame(x$ttest))
-  ab<-list()
-  symp<-list()
-  stars<-list()
-  for(i in 1:length(x$Pvalues)){
-    symp[[i]] <- stats::symnum(x$Pvalues[[i]], corr=FALSE,cutpoints = c(0,  .001,.01,.05, .1, 1), symbols = c("***","**","*","."," "))
-    stars[[i]]<-matrix(symp[[i]], nrow=length(x$Pvalues[[i]]))
-    ab[[i]]<-matrix(paste(x$coefficients[[i]],"(", x$StDev[[i]],")",stars[[i]], sep=""), nrow=length(x$StDev[[i]]))
-    dimnames(ab[[i]])<-dimnames(x$coefficients[[1]])
-  }
-  attributes(ab)<-attributes(x$coefficients)
-  x$stars<-stars
-  x$starslegend<-symp[[1]]
-  x$bigcoefficients<-ab
-  x$aic<-x$AIC
-  x$bic<-x$BIC
-  x$t <- t
-  x$k <- k
-  class(x) <- 'summary.VLSTAR'
-  return(x)
-  NextMethod('summary')
-}
-
-#' @S3method print summary.VLSTAR
-print.summary.VLSTAR<-function(x,...){
-  signif.stars ="show.signif.stars"
-  digits = 3
-  coeftoprint<-list()
-  myformat<-function(x,digits, toLatex=FALSE){
-    r<-x
-    littlex<-abs(x)<10^-(digits)
-    r[!littlex]<-formatC(x[!littlex],digits=digits, format="f")
-    r[littlex]<-format(x[littlex],digits=min(digits,2), scientific=TRUE)
-    if(toLatex)
-      r<-gsub("(e.*)","slashtext{\\1}",r)
-    if(class(x)=="numeric")
-      return(noquote(r))
-    if(class(x)=="matrix")
-      return(matrix(noquote(r), ncol=ncol(x), nrow=nrow(x)))
-  }
-  for(i in 1:length(x$bigcoefficients)){
-    a<-myformat(x$coefficients[[i]], digits)
-    b<-myformat(x$StDev[[i]], digits)
-    c<-myformat(x$Pvalues[[i]], digits)
-    aic1 <- round(x$AIC,2)
-    bic1 <- round(x$BIC,2)
-    if(signif.stars == getOption("show.signif.stars")){
-      stars1<-x$stars[[i]]
-    }else{
-      stars1<-NULL
-    }
-    #coeftoprint[[i]]<-matrix(c(paste(a,"(", b,")",stars, sep=""), '', round(x$Cgamma[i,1],4), round(x$Cgamma[i,2],4),'',
-    #                          aic1[i], bic1[i]), nrow=(length(x$StDev[[1]])+6))
-    coeftop <- cbind(a, b, c, stars1)
-    colnames(coeftop) <- c('Estimate', 'Std. Error', 'p-value', '')
-    coeftoprint[[i]] <- rbind(coeftop, rep('', 4), c(round(x$Cgamma[i,1],4),'','',''), c(round(x$Cgamma[i,2],4),'','',''),
-                              rep('', 4), c(aic1[i],'','',''), c(bic1[i], '','',''))
-    rownames(coeftoprint[[i]])<- c(rownames(x$Bhat),"---",'gamma', 'c',"---", 'AIC', 'BIC')
-  }
-  names(coeftoprint) <- colnames(x$Bhat)
-  cat("Model VLSTAR with ", x$m, " regimes\n", sep ='')
-  cat("\nFull sample size:",x$T, "\tEnd sample size:", x$t)
-  cat("\nNumber of variables:", x$k,"\tNumber of estimated parameters:", x$npar)
-  #cat("\nAIC",x$aic)
-  #cat("\nBIC", x$bic)
-  cat("\nMultivariate log-likelihood:", x$MultiLL,"\n\n")
-  cat('\nCoefficients:')
-  print(noquote(coeftoprint))
-  if (signif.stars)
-    cat("---\nSignif. codes: ", attr(x$starslegend, "legend"), "\n")
-  #cat("\nThreshold values:",x$Cgamma[,2])
-  NextMethod('print.summary')
-}
-
